From 8002a0b3cfa6380378417cda6485c165b911deac Mon Sep 17 00:00:00 2001
From: akira <marivalenm@gmail.com>
Date: Sat, 4 Jul 2015 21:15:31 +0200
Subject: [PATCH] Honour SOURCE_DATE_EPOCH environment variable for
 reproducible output

If the environment variable SOURCE_DATE_EPOCH is set then the HTML, LaTeX and Man page output will contain the timestamp from the variable instead of the current date. The content of this variable is the unix epoch. This is useful for the Debian reproducible builds project [1] because it will cause packages to become reproducible while allowing them to have a timestamp.

[1] https://wiki.debian.org/ReproducibleBuilds
---
 src/util.cpp | 38 +++++++++++++++++++++++++++++++++++++-
 1 file changed, 37 insertions(+), 1 deletion(-)

diff --git a/src/util.cpp b/src/util.cpp
index 3ee7ae5..9413347 100644
--- a/src/util.cpp
+++ b/src/util.cpp
@@ -18,6 +18,7 @@
 #include <ctype.h>
 #include <errno.h>
 #include <math.h>
+#include <limits.h>
 
 #include "md5.h"
 
@@ -2471,7 +2472,42 @@ QCString fileToString(const char *name,bool filter,bool isSourceCode)
 
 QCString dateToString(bool includeTime)
 {
-  QDateTime current = QDateTime::currentDateTime();
+  char *source_date_epoch;
+  unsigned long long epoch;
+  char *endptr;
+  QDateTime current;
+  source_date_epoch = getenv("SOURCE_DATE_EPOCH");
+  if (source_date_epoch)
+  {
+    errno = 0;
+    epoch = strtoull(source_date_epoch, &endptr, 10);
+    if ((errno == ERANGE && (epoch == ULLONG_MAX || epoch == 0))
+		|| (errno != 0 && epoch == 0))
+    {
+	err("Environment variable $SOURCE_DATE_EPOCH: strtoull: %s\n", strerror(errno));
+	exit(EXIT_FAILURE);
+    }
+    if (endptr == source_date_epoch)
+    {
+	err("Environment variable $SOURCE_DATE_EPOCH: No digits were found: %s\n", endptr);
+	exit(EXIT_FAILURE);
+    }
+    if (*endptr != '\0')
+    {
+	err("Environment variable $SOURCE_DATE_EPOCH: Trailing garbage: %s\n", endptr);
+	exit(EXIT_FAILURE);
+    }
+    if (epoch > UINT_MAX)
+    {
+	err("Environment variable $SOURCE_DATE_EPOCH: value must be smaller than or equal to: %u but was found to be: %llu \n", UINT_MAX  ,epoch);
+	exit(EXIT_FAILURE);
+    }
+    current.setTime_t(epoch);
+  }
+  else
+  {
+    current = QDateTime::currentDateTime();
+  }
   return theTranslator->trDateTime(current.date().year(),
                                    current.date().month(),
                                    current.date().day(),
-- 
2.1.1

